package io.terminus.monitor;

import android.content.Context;
import android.text.TextUtils;
import android.util.Log;

import org.litepal.LitePal;

import java.util.HashMap;
import java.util.Map;

import io.terminus.monitor.model.BaseModel;
import io.terminus.monitor.model.DeviceInfo;
import io.terminus.monitor.model.Enum.UploadModel;
import io.terminus.monitor.model.ErrorModel;
import io.terminus.monitor.model.EventModel;
import io.terminus.monitor.model.PageModel;
import io.terminus.monitor.model.RequestModel;

import io.terminus.monitor.model.PerformanceModel;
import io.terminus.monitor.module.LitePalHelper;
import io.terminus.monitor.module.UploadManager;
import io.terminus.monitor.util.DeviceUtil;
import io.terminus.monitor.util.ToolUtil;

/**
 * User : yh
 * Date : 17/2/21
 */

public class RNMobClickAgent {
    private static final String TAG = "TerminusMobClick";

    private static RNMobClickAgent agent;
    private Map<String,PageModel> pageInfo;

    private RNMobClickAgent(){
        pageInfo = new HashMap<>();
    }

    private static Context mContext;

    public static RNMobClickAgent getInstance(){
        if(agent == null){
            agent = new RNMobClickAgent();
        }
        return agent;
    }

    public void init(Context mContext){
        RNMobClickAgent.mContext = mContext;
        LitePal.initialize(mContext);
        LitePalHelper.getInstance().LogModelRegist(DeviceInfo.class);
        LitePalHelper.getInstance().LogModelRegist(ErrorModel.class);
        LitePalHelper.getInstance().LogModelRegist(EventModel.class);
        LitePalHelper.getInstance().LogModelRegist(PageModel.class);
        LitePalHelper.getInstance().LogModelRegist(PerformanceModel.class);
        LitePalHelper.getInstance().LogModelRegist(RequestModel.class);

        UploadManager.getInstance().init(mContext);
        AnalyticsConfig.setDeviceId(DeviceUtil.getDeviceId(mContext));
    }

    public void setAppKey(String appkey){
        //// TODO: 17/2/21
        AnalyticsConfig.setAppKey(appkey);
      UploadManager.getInstance().addDeviceTask(DeviceUtil.getDeviceInfo(RNMobClickAgent.mContext));

    }

    public void setChannel(String channel){
        //// TODO: 17/2/21
    }


    public void onEvent(BaseModel event){
        UploadManager.getInstance().addTask(modleAppendInfo(event));
    }


    public void setUploadModel(UploadModel model){
        AnalyticsConfig.setUploadModel(model);
    }

    public void setSignin(String userId){
        AnalyticsConfig.setUserId(userId);
    }

    private BaseModel modleAppendInfo(BaseModel model){
        if(!TextUtils.isEmpty(AnalyticsConfig.getUserId())){
            model.setUid(AnalyticsConfig.getUserId());
        }
        model.setDate(ToolUtil.getNowTime());
        return model;
    }

    public void onPageStart(String pageName,String lastPage){
        if(TextUtils.isEmpty(pageName)){
            Log.e(TAG,"pageName is empty");
            return;
        }
        PageModel model = new PageModel();
        model.setDt(pageName);
        model.setDr(lastPage);
        model.setSt(ToolUtil.getNowTimeMillis());
        pageInfo.put(pageName,model);
    }

    public void onPageEnd(String pageName){
        if(!pageInfo.containsKey(pageName) ){
            Log.e(TAG,pageName+" not exit");
            return;
        }
        PageModel model = pageInfo.get(pageName);
        model.setTp(String.valueOf((ToolUtil.getNowTimeMillis() - model.getSt())/1000));
        onEvent(model);
        pageInfo.remove(pageName);
    }

    public Context getContext(){
        return mContext;
    }

}
