
package io.terminus.monitor;

import android.support.annotation.UiThread;
import android.view.Choreographer;

import com.facebook.react.bridge.LifecycleEventListener;
import com.facebook.react.bridge.NativeModuleCallExceptionHandler;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.ReadableNativeMap;
import com.facebook.react.bridge.UiThreadUtil;
import com.facebook.react.modules.debug.FpsDebugFrameCallback;
import com.facebook.react.modules.core.ChoreographerCompat;

import com.google.gson.Gson;

import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import io.terminus.monitor.model.Enum.UploadModel;
import io.terminus.monitor.model.ErrorModel;
import io.terminus.monitor.model.EventModel;
import io.terminus.monitor.model.PageModel;
import io.terminus.monitor.module.CrashHandler;
import io.terminus.monitor.module.PerFormanceTool;
import io.terminus.monitor.util.ToolUtil;
import  io.terminus.monitor.model.RequestModel;

public class RNMonitorModule extends ReactContextBaseJavaModule implements LifecycleEventListener{

    private final ReactApplicationContext reactContext;
    private FpsDebugFrameCallback callback;
    private Timer timer ;

    private static final String WIFI_LIMIT = "WIFI_LIMIT";
    private static final String REAL_TIME_WIFI = "REAL_TIME_WIFI";
    private static final String REAL_TIME = "REAL_TIME";

    public RNMonitorModule(ReactApplicationContext reactContext) {
        super(reactContext);
        this.reactContext = reactContext;
        RNMobClickAgent.getInstance().init(reactContext.getApplicationContext());
        PerFormanceTool.getInstance().init(reactContext);
    }

    @ReactMethod
    public void init(String appkey, int model) {
        RNMobClickAgent.getInstance().setAppKey(appkey);
        UploadModel uModel = UploadModel.getModel(model);
        RNMobClickAgent.getInstance().setUploadModel(uModel);
        UiThreadUtil.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (callback == null) {
                    callback = new FpsDebugFrameCallback(reactContext);
                    callback.start();
                }
            }
        });
        if(timer != null){
            return;
        }
//        timer = new Timer();
//        timer.schedule(new TimerTask() {
//            @Override
//            public void run() {
//                UiThreadUtil.runOnUiThread(new Runnable() {
//                    @Override
//                    public void run() {
//                        if (callback != null) {
//                            PerFormanceTool.getInstance().setFPSsetFPS(callback.getFPS(), callback.getJSFPS());
//                        }
//                    }
//                });
//            }
//        }, 3000, 3000);
        reactContext.addLifecycleEventListener(this);
    }

    @ReactMethod
    public Map<String, Object> getConstants() {
        final Map<String, Object> constants = new HashMap<>();
        constants.put(REAL_TIME, UploadModel.REAL_TIME.model);
        constants.put(WIFI_LIMIT, UploadModel.WIFI_LIMIT.model);
        constants.put(REAL_TIME_WIFI,UploadModel.REAL_TIME_WIFI.model);
        return constants;
    }

    @ReactMethod
    public void onSignIn(String userId) {
        RNMobClickAgent.getInstance().setSignin(userId);
    }

    @ReactMethod
    public void onCusEvent(ReadableMap map) {
        ReadableNativeMap nativeMap = (ReadableNativeMap) map;
        String event_str = new Gson().toJson(nativeMap.toHashMap());
        EventModel eventModel = new Gson().fromJson(event_str, EventModel.class);
        RNMobClickAgent.getInstance().onEvent(eventModel);
    }


    @ReactMethod
    public void sendRequest(ReadableMap map) {
        ReadableNativeMap nativeMap = (ReadableNativeMap) map;
        String event_str = new Gson().toJson(nativeMap.toHashMap());
        RequestModel requestModel = new Gson().fromJson(event_str, RequestModel.class);
        RNMobClickAgent.getInstance().onEvent(requestModel);
    }


    @ReactMethod
    public void onError(ReadableMap map) {
        ReadableNativeMap nativeMap = (ReadableNativeMap) map;
        String event_str = new Gson().toJson(nativeMap.toHashMap());
        ErrorModel errorModel = new Gson().fromJson(event_str, ErrorModel.class);
        RNMobClickAgent.getInstance().onEvent(errorModel);
    }

    @ReactMethod
    public void onPageStart(String pageName,String lastPage) {
        RNMobClickAgent.getInstance().onPageStart(pageName,lastPage);
    }

    @ReactMethod
    public void onPageEnd(String pageName) {
        RNMobClickAgent.getInstance().onPageEnd(pageName);
    }



    @ReactMethod
    public void pageLoadTime(String pageName,String pageTime) {
        PerFormanceTool.getInstance().pageLoadTime(pageName,pageTime);
    }


    @Override
    public String getName() {
        return "RNMonitor";
    }

    @Override
    public void onHostResume() {

    }

    @Override
    public void onHostPause() {

    }

    @Override
    public void onHostDestroy() {
        if(timer != null ){
            timer.cancel();
            timer = null;
        }
    }
}