package io.terminus.monitor.module;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Build;
import android.text.TextUtils;
import android.widget.Toast;

import com.google.gson.Gson;
import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;
import com.loopj.android.http.ResponseHandlerInterface;
import com.loopj.android.http.SyncHttpClient;
import com.loopj.android.http.PreemptiveAuthorizationHttpRequestInterceptor;

import java.util.ArrayList;


import io.terminus.monitor.AnalyticsConfig;
import io.terminus.monitor.RNMobClickAgent;
import io.terminus.monitor.model.BaseModel;
import io.terminus.monitor.model.DeviceInfo;
import io.terminus.monitor.model.Enum.NetInfoEnum;
import io.terminus.monitor.model.Enum.UploadModel;
import io.terminus.monitor.util.DeviceIDUtil;
import io.terminus.monitor.util.DeviceUtil;
import io.terminus.monitor.util.ToolUtil;

/**
 * User : yh
 * Date : 17/2/21
 */

public class UploadManager {
    private static UploadManager manager;
    private ArrayList<BaseModel> taskQueue;
    private TaskThread taskThread;
    private ConnectionChangeReceiver receiver;
    private NetInfoEnum netState;
    private SyncHttpClient netClient;
    private final int LIMIT = 20;
    private Context cusmcontext;

    private UploadManager(){
        taskQueue = new ArrayList<BaseModel>();
        taskThread = new TaskThread();
        taskThread.start();
    }

    public static UploadManager getInstance(){
        if(manager == null){
            manager = new UploadManager();
        }
        return manager;
    }

    public void init(Context mcontext){
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction("android.net.conn.CONNECTIVITY_CHANGE");
        receiver = new ConnectionChangeReceiver();
        mcontext.registerReceiver(receiver,intentFilter);
        netState = DeviceUtil.getNetWorkInfo(mcontext);
        netClient = new SyncHttpClient();
        netClient.addHeader("User-Agent","Android");
        netClient.addHeader("Referer","home_page");
        cusmcontext = mcontext;

        synchronized (taskQueue) {
            taskQueue.addAll(LitePalHelper.getInstance().getAllData());
            taskQueue.notifyAll();
        }
    }

    public void LoadLogFromDB(){
        synchronized (taskQueue) {
            taskQueue.addAll(LitePalHelper.getInstance().getAllData());
            if(!taskQueue.isEmpty()){
                taskQueue.notifyAll();
            }
        }
    }

     public void addTask(BaseModel model){
        LitePalHelper.getInstance().save(applyBaseParam(model));

        if(AnalyticsConfig.getUploadModel() == UploadModel.REAL_TIME){
            synchronized (taskQueue) {
                taskQueue.addAll(LitePalHelper.getInstance().getAllData());
                taskQueue.notifyAll();
            }
        }else if((AnalyticsConfig.getUploadModel() == UploadModel.WIFI_LIMIT || AnalyticsConfig.getUploadModel() == UploadModel.REAL_TIME_WIFI)&& netState == NetInfoEnum.WIFI){
            synchronized (taskQueue){
                if(LitePalHelper.getInstance().count() >= LIMIT){
                    taskQueue.addAll(LitePalHelper.getInstance().getAllData());
                    taskQueue.notifyAll();
                }
            }

        }
    }


    public void addDeviceTask(BaseModel model){
        LitePalHelper.getInstance().save(applyBaseParam(model));
        synchronized (taskQueue) {
                taskQueue.addAll(LitePalHelper.getInstance().getAllData());
                taskQueue.notifyAll();
            }
    }

    public BaseModel applyBaseParam(BaseModel model){
        model.setDate(ToolUtil.getNowTime());
        model.setNs(DeviceUtil.getNetWorkInfo(cusmcontext).getStatus());
        model.setAv(DeviceUtil.getAppVersion(cusmcontext));
        model.setUid(AnalyticsConfig.getUserId());
        model.setAk(AnalyticsConfig.getAppKey());
        DeviceIDUtil DeviceID = new DeviceIDUtil(cusmcontext);
        model.setVid(DeviceID.getDeviceUUId().toString());
        model.setCid(DeviceID.getDeviceId());
        model.setDh(cusmcontext.getPackageName());
        model.setBr(Build.MODEL);
        model.setOsv(Build.VERSION.RELEASE);
        model.setMd(Build.MODEL);
        String osName =  Build.MANUFACTURER + " android";
        model.setOsn(osName);
        model.setGps(DeviceUtil.locationInfo(cusmcontext));
        model.setUa("Android");
        return model;
    }




    private class TaskThread extends Thread{
        @Override
        public void run() {
            while (true){
                synchronized (taskQueue){
                    if(taskQueue.isEmpty()){
                        try {
                            taskQueue.wait();
                        }catch (Exception e){

                        }
                    }else{
                        RequestParams params = new RequestParams();
                        params.put("ak",AnalyticsConfig.getAppKey());
                        params.put("cid",AnalyticsConfig.getDeviceId());
                        params.put("ip",DeviceUtil.getNetIp());
                        params.put("uid",AnalyticsConfig.getUserId());
                        for(BaseModel model : taskQueue){
                            LitePalHelper.getInstance().deleteData(model.getClass(),model.getId());
                            params.add("data",ToolUtil.toRequestParams(model).toString());
                        }
                        taskQueue.clear();
                        netClient.post(AnalyticsConfig.getDomain(), params, new AsyncHttpResponseHandler() {
                            @Override
                            public void onSuccess(int statusCode, org.apache.http.Header[] headers, byte[] responseBody) {

                            }

                            @Override
                            public void onFailure(int statusCode, org.apache.http.Header[] headers, byte[] responseBody, Throwable error) {

                            }
                        });
                    }
                }
            }
        }
    }

    public class ConnectionChangeReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {
            netState = DeviceUtil.getNetWorkInfo(context);
//            if(netState == NetInfoEnum.WIFI && AnalyticsConfig.getUploadModel() == UploadModel.REAL_TIME){
//                LoadLogFromDB();
//            }
        }
    }
}
