package io.terminus.monitor.util;

import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Build;
import android.os.StatFs;
import android.telephony.TelephonyManager;
import android.text.format.Formatter;
import android.util.DisplayMetrics;

import com.google.gson.Gson;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.List;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import io.terminus.monitor.model.DeviceInfo;
import io.terminus.monitor.model.Enum.NetInfoEnum;
import io.terminus.monitor.util.DeviceIDUtil;
import android.os.Environment;

import static android.content.Context.TELEPHONY_SERVICE;

/**
 * User : yh
 * Date : 17/2/20
 */

public class DeviceUtil {

    /**
     * 获取设备信息
     * @param mContext
     * @return
     */
    public static DeviceInfo getDeviceInfo(Context mContext){
        DeviceInfo info = new DeviceInfo();
        DisplayMetrics dm = mContext.getResources().getDisplayMetrics();
        info.setSr(dm.widthPixels+"*"+dm.heightPixels);
        info.setCh(getChannel(mContext));
        info.setJb((ToolUtil.root() ? 1 : 0));
        File dataFileDir=Environment.getDataDirectory();
        File path = Environment.getExternalStorageDirectory();
        info.setRom(getMemoryInfo(dataFileDir,mContext));
        info.setMem(getTotalRam(mContext));
        info.setSd(getMemoryInfo(path,mContext));
        info.setCpu(Build.CPU_ABI);
        info.setSdk(Build.VERSION.RELEASE);
        return info;
    }

    //渠道信息
    public static String getChannel(Context mContext){
        try{
            ApplicationInfo info = mContext.getPackageManager().getApplicationInfo(mContext.getPackageName(), PackageManager.GET_META_DATA);
            return info.metaData.getString("CHANNEL");
        }catch (Exception e){
            e.printStackTrace();
        }
        return null;
    }

    public static String getAppVersion(Context mContext){
        try {
            PackageInfo info = mContext.getPackageManager().getPackageInfo(mContext.getPackageName(),0);
            return info.versionName;
        }catch (Exception e) {
            //
        }
        return null;
    }

    public static NetInfoEnum getNetWorkInfo(Context mContext){
        NetworkInfo networkInfo = ((ConnectivityManager)mContext.getSystemService(Context.CONNECTIVITY_SERVICE)).getActiveNetworkInfo();
//        networkInfo.
        if(networkInfo == null || !networkInfo.isConnected()){
            return NetInfoEnum.OFFLINE;
        }
        if(networkInfo.getType() == ConnectivityManager.TYPE_WIFI){
            return NetInfoEnum.WIFI;
        }
        if(networkInfo.getType() != ConnectivityManager.TYPE_MOBILE){
            return NetInfoEnum.UNKNOWN;
        }
        String subTypeName = networkInfo.getSubtypeName();
        int subType = networkInfo.getSubtype();
        NetInfoEnum type = NetInfoEnum.UNKNOWN;
        switch (subType){
            case TelephonyManager.NETWORK_TYPE_GPRS:
            case TelephonyManager.NETWORK_TYPE_EDGE:
            case TelephonyManager.NETWORK_TYPE_CDMA:
            case TelephonyManager.NETWORK_TYPE_1xRTT:
            case TelephonyManager.NETWORK_TYPE_IDEN: //api<8 : replace by 11
                type = NetInfoEnum.G2;
                break;
            case TelephonyManager.NETWORK_TYPE_UMTS:
            case TelephonyManager.NETWORK_TYPE_EVDO_0:
            case TelephonyManager.NETWORK_TYPE_EVDO_A:
            case TelephonyManager.NETWORK_TYPE_HSDPA:
            case TelephonyManager.NETWORK_TYPE_HSUPA:
            case TelephonyManager.NETWORK_TYPE_HSPA:
            case TelephonyManager.NETWORK_TYPE_EVDO_B: //api<9 : replace by 14
            case TelephonyManager.NETWORK_TYPE_EHRPD:  //api<11 : replace by 12
            case TelephonyManager.NETWORK_TYPE_HSPAP:  //api<13 : replace by 15
                type = NetInfoEnum.G3;
                break;
            case TelephonyManager.NETWORK_TYPE_LTE:    //api<11 : replace by 13
                type = NetInfoEnum.G4;
                break;
            default:
                if(subTypeName.equalsIgnoreCase("TD-SCDMA") || subTypeName.equalsIgnoreCase("WCDMA") || subTypeName.equalsIgnoreCase("CDMA2000")){
                    type = NetInfoEnum.G3;
                }
                break;
        }
        return type;
    }

    public static String locationInfo(Context mContext){
        LocationManager locationManager = (LocationManager) mContext.getSystemService(Context.LOCATION_SERVICE);
        List<String> providers = locationManager.getProviders(true);
        String locationProvider;
        if(providers.contains(LocationManager.GPS_PROVIDER)){
            //如果是GPS
            locationProvider = LocationManager.GPS_PROVIDER;
        }else if(providers.contains(LocationManager.NETWORK_PROVIDER)){
            //如果是Network
            locationProvider = LocationManager.NETWORK_PROVIDER;
        }else{
            return null;
        }
        Location location = locationManager.getLastKnownLocation(locationProvider);
        if(location == null){
            return null;
        }
        return location.getLatitude()+","+location.getLongitude();
    }

    public static String getDeviceId(Context mContext){
        TelephonyManager tm = (TelephonyManager) mContext.getSystemService(TELEPHONY_SERVICE);
        final String tmDevice;
        tmDevice = "" +  new DeviceIDUtil(mContext).getDeviceId();
        return tmDevice;
    }


    public static String getNetIp() {
        URL infoUrl = null;
        InputStream inStream = null;
        String ipLine = "";
        HttpURLConnection httpConnection = null;
        try {
            infoUrl = new URL("http://pv.sohu.com/cityjson?ie=utf-8");
            URLConnection connection = infoUrl.openConnection();
            httpConnection = (HttpURLConnection) connection;
            int responseCode = httpConnection.getResponseCode();
            if (responseCode == HttpURLConnection.HTTP_OK) {
                inStream = httpConnection.getInputStream();
                BufferedReader reader = new BufferedReader(new InputStreamReader(inStream, "utf-8"));
                StringBuilder strber = new StringBuilder();
                String line = null;
                while ((line = reader.readLine()) != null){
                    strber.append(line + "\n");
                }
                Pattern pattern = Pattern
                        .compile("((?:(?:25[0-5]|2[0-4]\\d|((1\\d{2})|([1-9]?\\d)))\\.){3}(?:25[0-5]|2[0-4]\\d|((1\\d{2})|([1-9]?\\d))))");
                Matcher matcher = pattern.matcher(strber.toString());
                if (matcher.find()) {
                    ipLine = matcher.group();
                }
            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                inStream.close();
                httpConnection.disconnect();
            } catch (IOException e) {
                e.printStackTrace();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
        return ipLine;
    }


    public static String getMemoryInfo(File path,Context mContext) {
        long blockSize;
        long totalBlockCount;
        StatFs stat = new StatFs(path.getPath());
        long blockSiza= stat.getBlockSize();
        long totalBlocks = stat.getBlockCount();

        //检测系统版本
        if(Build.VERSION.SDK_INT >=Build.VERSION_CODES.JELLY_BEAN_MR2){

            //获取每个扇区的大小
            blockSize = stat.getBlockSizeLong();
            //获取总共有多少扇区
            totalBlockCount=stat.getBlockCountLong();

        }else{
            blockSize=stat.getBlockSize();
            totalBlockCount=stat.getBlockCount();
        }

        // 磁盘总大小
        String totalMemory =Formatter.formatFileSize(mContext, blockSize*totalBlockCount);

        return totalMemory;
    }

    /** * 获取SD卡信息 * * @return SDCardInfo */

    /** * 判断SD卡是否可用 * * @return true : 可用<br>false : 不可用 */
    public static boolean isSDCardEnable() {
        return Environment.MEDIA_MOUNTED.equals(Environment.getExternalStorageState());
    }


    // 获取内存大小
    public static String getTotalRam(Context context){
        String path = "/proc/meminfo";
        String firstLine = null;
        int totalRam = 0 ;
        try{
            FileReader fileReader = new FileReader(path);
            BufferedReader br = new BufferedReader(fileReader,8192);
            firstLine = br.readLine().split("\\s+")[1];
            br.close();
        }catch (Exception e){
            e.printStackTrace();
        }
        if(firstLine != null){
            totalRam = (int)Math.ceil((new Float(Float.valueOf(firstLine) / (1024 * 1024)).doubleValue()));
        }

        return totalRam + "GB";//返回1GB/2GB/3GB/4GB
    }




    public static String getCpuName() {
        try {
            FileReader fr = new FileReader("/proc/cpuinfo");
            BufferedReader br = new BufferedReader(fr);
            String text = br.readLine();
            String[] array = text.split(":\\s+", 2);
            for (int i = 0; i < array.length; i++) {
            }
            return array[1];
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }
}

