import React from 'react'
import { NativeModules } from 'react-native';
import * as MonitorType from './MonitorType';
const { RNMonitor } = NativeModules;

var Monitor  = {    
    /**
     * 统计初始化，
     * appkey 系统分配的
     * UploadModel  上传方式 ; REAL_TIME : 实时上传 READL_TIME_WIFI ：WiFi下实时上传 WIFI : WiFi下首次启动上传
     */
    init(appkey,UploadModel){
        RNMonitor.init(appkey,UploadModel);
        // require('ErrorUtils').setGlobalHandler((error)=>{
        //     RNMonitor.onError({erm:error.message,sta:error.stack})
        // })
        // this.getNowFormatDate = this.getNowFormatDate.bind(this);
    },

    /***
     * 用户统计
     */
    onSignIn(userId){
      RNMonitor.onSignIn(userId);
    },
    
    /**
     * event 数据结构
     * {
     *   eventPageName  事件所在页面
     *   eventId,       事件id
     *   createTime,    事件发生时间  //可不传
     *   eventName,     事件名称
     *   extra, 
     * }
     */
    onCusEvent(obj){
        if(!obj){
          throw new Error("event info should not be null")
          return;
        }
        var keys = ["eventId","eventName","eventPageName","extra","createTime"];
        let event = {};
        keys.map((key,index)=>{
            if(key === 'eventId' && !obj[key]){
                throw new Error('eventId should not be null');
                return;
            }
            event[key] = obj[key];
        })
        if(event.createTime === undefined){
            event['createTime'] = this.getNowFormatDate();
        }
        RNMonitor.onCusEvent(event);

    },
    /**
     * 组件 willmount 方法中调用
     * pageObj 为当前页面或组件对象(this)，该名称必须与onPageEnd的参数名称保持一致，否则无法正确统计页面停留时间
     */
    onPageStart(pageName,lastPage){
        // const pageName = pageObj &&
        //                  pageObj._reactInternalInstance &&
        //                  pageObj._reactInternalInstance._currentElement &&
        //                  pageObj._reactInternalInstance._currentElement.type &&
        //                  pageObj._reactInternalInstance._currentElement.type.name
        if(pageName){
          RNMonitor.onPageStart(pageName,lastPage)
        }
    },
    
    /**
     * pageObj ，或者 页面发生跳转的时候调用
     */
    onPageEnd(pageName){
        // const pageName = pageObj &&
        //                  pageObj._reactInternalInstance &&
        //                  pageObj._reactInternalInstance._currentElement &&
        //                  pageObj._reactInternalInstance._currentElement.type &&
        //                  pageObj._reactInternalInstance._currentElement.type.name
        if(pageName){
          RNMonitor.onPageEnd(pageName)            
        }
    },
    /**
     * pageObj ，或者 页面发生跳转的时候调用
     */
    pageLoad(pageName,loadTime){
        if(pageName && loadTime ){
          RNMonitor.pageLoadTime(pageName,loadTime)            
        }
    },

   /*   requestParam:中必须包含参数：
    *
    请求响应时间  tt;
    请求路径  url;
    响应状态 st;
    请求方法 me;
    请求体大小 req;
    响应体大小 res;
    */
    sendRequest(requestParam){
        if(requestParam){
          RNMonitor.sendRequest(requestParam)            
        }
    },

    /*   eventParam:中必须包含参数：
    *
    target xpath  xp;
    事件坐标x  x;
    事件坐标y y;
    */
     onEvent(eventParam){
        if(eventParam){
          RNMonitor.event(eventParam)            
        }
    },
    /*   errorParam:中必须包含参数：
    *
    错误日志  erm;
    堆栈信息 sta;
    */
     onError(errorParam){
        if(errorParam){
          RNMonitor.onError(errorParam)            
        }
    },

    getNowFormatDate(){
      var date = new Date();
      var seperator1 = "/";
      var seperator2 = ":";
      var month = date.getMonth() + 1;
      var strDate = date.getDate();
      if (month >= 1 && month <= 9) {
        month = "0" + month;
      }
      if (strDate >= 0 && strDate <= 9) {
       strDate = "0" + strDate;
      }
      var currentdate = date.getFullYear() + seperator1 + month + seperator1 + strDate
            + " " + date.getHours() + seperator2 + date.getMinutes()
            + seperator2 + date.getSeconds();
      return currentdate;
    }
}



export { Monitor,MonitorType};
