
#import "RNMonitor.h"
#import "RNMonitorConfig.h"
#import "RNCusEventManager.h"
#import "RNPageManager.h"
#import "RNRequestManager.h"
#import "RNPerformanceManager.h"
#import "RNEventManager.h"
#import "RNLogManager.h"
@implementation RNMonitor

- (NSDictionary *)constantsToExport
{
    return @{ @"REAL_TIME": @"1", @"REAL_TIME_WIFI": @"2",@"WIFI":@"3"};
}

- (dispatch_queue_t)methodQueue
{
    return dispatch_get_main_queue();
}

RCT_EXPORT_MODULE(RNMonitor)

RCT_EXPORT_METHOD(init:(NSString*)appkey model:(NSString*)uploadModel){
    NSInteger type = uploadModel.integerValue;
    [[RNMonitorConfig shareInstance] startMonitorWithAppKey:appkey];
    switch (type) {
        case 1:
            [[RNMonitorConfig shareInstance] setType:RNUploadTypeRealTime];
            break;
        case 2:
            [[RNMonitorConfig shareInstance] setType:RNUploadTypeWifiLimit];
            break;
        case 3:
            [[RNMonitorConfig shareInstance] setType:RNUploadTypeActivate];
            break;
        default:
            break;
    }
    
}
RCT_EXPORT_METHOD(onSignIn:(NSString*)uid){
    
    [[RNMonitorConfig shareInstance] setUid:uid];
    
}

RCT_EXPORT_METHOD(onCusEvent:(NSDictionary*)event){
    NSArray*keys = @[@"eventId",@"eventName",@"eventPageName",@"extra"];
    NSMutableArray *values = [NSMutableArray arrayWithCapacity:4];
    for (NSString *key in keys) {
        [event.allKeys containsObject:key]?[values addObject:event[key]]: [values addObject:@""];
    }
    [[RNCusEventManager shareInstance] event:values[0]
                                eventName:values[1]
                                 pageName:values[2]
                                    extra:values[3]];
    
    
}



RCT_EXPORT_METHOD(onPageStart:(NSString*)pageName lastPage:(NSString*)lastPage){
 
    [[RNPageManager shareInstance] startPageView:pageName lastPage:lastPage];
    
}
RCT_EXPORT_METHOD(onPageEnd:(NSString*)pageName){
    
    [[RNPageManager shareInstance] endPageView:pageName];
}

RCT_EXPORT_METHOD(pageLoadTime:(NSString*)pageName pageTime:(NSString*)time){
    
    [[RNPerformanceManager shareInstance] pageLoadTime:pageName
                                                  time:time];
}

/*   requestParam:中必须包含参数：
 *
  请求响应时间  tt;
  请求路径  url;
  响应状态 st;
  请求方法 me;
  请求体大小 req;
  响应体大小 res;
*/
RCT_EXPORT_METHOD(sendRequest:(NSDictionary*)requestParam){
    [[RNRequestManager shareInstance] sendRequest:requestParam];
}
/*   eventParam:中必须包含参数：
 *
 target xpath  xp;
 事件坐标x  x;
 事件坐标y y;
 */
RCT_EXPORT_METHOD(event:(NSDictionary*)eventParam){
    [[RNEventManager shareInstance] event:eventParam];
}


/*   errorParam:中必须包含参数：
 *
 错误日志  erm;
 堆栈信息 sta;
 */
RCT_EXPORT_METHOD(onError:(NSDictionary*)errorParam){
    [[RNLogManager shareInstance] handerReactNativeError:errorParam];
}

@end
  
