//
//  RNMonitorUploadManager.m
//  RNMonitor
//
//  Created by zhiyu on 17/3/13.
//  Copyright © 2017年 Facebook. All rights reserved.
//

#import "RNMonitorUploadManager.h"
#import "RNDataManager.h"
#import "RNUtil.h"
#import "RNMonitorConfig.h"
#import "RNUpdateService.h"
#import "RNDevice.h"
#import "RNParamDO.h"
#import "RNFMDB.h"
#import "NSObject+RNMIU_ToDictionary.h"
#import "RNLocationManager.h"
@implementation RNMonitorUploadManager

+ (void)uploadMonitor:(NSString *)topic
{
    NSArray *array = [[RNFMDB shareDatabase] jq_lookupTable:topic
                                                 dicOrModel:[NSDictionary class]
                                                whereFormat:@"where t = '%@'",topic];
    
    if ([RNMonitorConfig shareInstance].uploadLimit && (array.count >= [RNMonitorConfig shareInstance].uploadLimit)) {
        NSArray *newArray = [self recombinationArray:array length:[RNMonitorConfig shareInstance].uploadLimit];
        [newArray enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            [RNMonitorUploadManager upload:obj table:topic];
        }];
    }else if(![RNMonitorConfig shareInstance].uploadLimit){
        [RNMonitorUploadManager upload:array table:topic];
    }
    
}

+ (void)applyBaseParam:(NSString *)topic
              pageName:(NSString*)pageName
                  data:(RNParamDO*)paramDO {
    paramDO.date = [RNUtil getCurrentTime];
    paramDO.ns = [RNDevice iphoneNetState];
    paramDO.av = [RNDevice appVersion];
    paramDO.uid = [RNMonitorConfig shareInstance].uid;
    paramDO.ak = [RNMonitorConfig shareInstance].appKey;
    paramDO.dp = pageName;
    paramDO.dh = [RNDevice bundleId];
    paramDO.cid = [RNDevice deviceId];
    paramDO.br =@"iphone";
    paramDO.md = [RNDevice iphoneType];
    paramDO.osn = @"iOS";
    paramDO.osv = [RNDevice iphoneSystemVersion];
    paramDO.ua = @"IOS";
    paramDO.vid = [RNDevice deviceId];
    paramDO.t = topic;
    
    if([RNLocationManager shareInstance].location){
        paramDO.gps = [RNLocationManager shareInstance] .location;
        [RNMonitorUploadManager checkUploadForENV:paramDO table:topic];
    }else{
        [[RNLocationManager shareInstance] startLocation:^(NSString *location, NSError *error) {
            if (location) {
                paramDO.gps = location;
            }
            [RNMonitorUploadManager checkUploadForENV:paramDO table:topic];
        }];
    }
    
    
    
}


+ (void)checkUploadForENV:(id)data table:(NSString *)tableName{
    
    if ([tableName isEqualToString:KDeviceEntityName]) {
        [self upload:@[[data RNMIU_ToDictionary]] table:tableName];
        return ;
    }
    
    if([RNMonitorConfig shareInstance].Debug){
#if DEBUG
        [[RNFMDB shareDatabase] jq_insertTable:tableName dicOrModel:data];
        [self upload:@[[data RNMIU_ToDictionary]] table:tableName];
#else
        [self checkUpload:data table:tableName];
#endif
    }else{
        [self checkUpload:data table:tableName];
    }
    
}


+ (void)checkUpload:(id)data table:(NSString *)tableName{
    
    if([RNMonitorConfig shareInstance].type == RNUploadTypeRealTime){
        [[RNFMDB shareDatabase] jq_insertTable:tableName dicOrModel:data];
        [self upload:@[[data RNMIU_ToDictionary]] table:tableName];
    }else if([RNMonitorConfig shareInstance].type == RNUploadTypeWifiLimit ){
        [[RNFMDB shareDatabase] jq_insertTable:tableName dicOrModel:data];
        ![[RNDevice iphoneNetState] isEqualToString:@"wifi"]?: [self uploadMonitor:tableName];
    }else if([RNMonitorConfig shareInstance].type == RNUploadTypeActivate){
        [[RNFMDB shareDatabase] jq_insertTable:tableName dicOrModel:data];
    }
}


// 根据数据上传
+ (void)upload:(id)monData table:(NSString *)tableName{
    if (!monData || [(NSArray*)monData count] ==0) {
        return;
    }
    id param =@{@"cid":[RNUtil deviceId],
                @"ak":[RNMonitorConfig shareInstance].appKey?:@"",
                @"uid":[RNMonitorConfig shareInstance].uid?:@"",
                @"data":monData};
    
    [RNUpdateService POST:[RNMonitorConfig shareInstance].baseURL
               parameters:param
              resultBlock:^(BOOL success, id data, NSError *error) {
                  if (success) {
                      [monData enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
                          [[RNFMDB shareDatabase] jq_deleteTable:tableName whereFormat:@"where date = '%@'",obj[@"date"]];
                      }];
                  }
              }];
}

+ (NSArray *)recombinationArray:(NSArray*)array length:(NSInteger)subSize {
    unsigned long count = array.count % subSize == 0 ? (array.count / subSize) : (array.count / subSize + 1);
    NSMutableArray *arr = [[NSMutableArray alloc] init];
    
    for (int i = 0; i < count; i ++) {
        int index = i * subSize;
        NSMutableArray *arr1 = [[NSMutableArray alloc] init];
        [arr1 removeAllObjects];
        int j = index;
        while (j < subSize*(i + 1) && j < array.count) {
            [arr1 addObject:[array objectAtIndex:j]];
            j += 1;
        }
        [arr addObject:[arr1 copy]];
    }
    return [arr copy];
}
@end
