//
//  UncaughtExceptionHandler.m
//  iOSCrash
//
//  Created by wzy on 16/1/15.
//  Copyright © 2016年 wzy. All rights reserved.
//

#import "RNLogManager.h"
#include <libkern/OSAtomic.h>
#include <execinfo.h>
#import "RNMonitorConfig.h"
#import "RNDataManager.h"
#import "RNUtil.h"
#import "RNDevice.h"
#import "RNUpdateService.h"
#import "RNLogDO.h"
#import "NSObject+RNMIU_ToDictionary.h"
#import "RNFMDB.h"
#import "RNMonitorUploadManager.h"
#import "RCTAssert.h"
NSString *  KReceiveLogNotification = @"KReceiveLogNotification";

@interface RNLogManager ()<UIAlertViewDelegate>

@end
NSString * const UncaughtExceptionHandlerSignalExceptionName = @"UncaughtExceptionHandlerSignalExceptionName";
NSString * const UncaughtExceptionHandlerSignalKey = @"UncaughtExceptionHandlerSignalKey";
NSString * const UncaughtExceptionHandlerAddressesKey = @"UncaughtExceptionHandlerAddressesKey";

volatile int32_t UncaughtExceptionCount = 0;
const int32_t UncaughtExceptionMaximum = 10;

const NSInteger UncaughtExceptionHandlerSkipAddressCount = 4;
const NSInteger UncaughtExceptionHandlerReportAddressCount = 5;


@implementation RNLogManager{
    BOOL dismissed;
}
+ (instancetype)shareInstance {
    static RNLogManager *_instance;
    static dispatch_once_t once_t;
    dispatch_once(&once_t, ^{
        if (!_instance) {
            _instance = [[self alloc] init];
        }
    });
    return _instance;
}

- (void)start{
    [[RNFMDB shareDatabase] jq_createTable:KLogEntityName dicOrModel: [RNLogDO new]];
    InstallUncaughtExceptionHandler();
    
}




+ (NSArray *)backtrace
{
    void* callstack[128];
    int frames = backtrace(callstack, 128);
    char **strs = backtrace_symbols(callstack, frames);
    
    int i;
    NSMutableArray *backtrace = [NSMutableArray arrayWithCapacity:frames];
    for (
         i = UncaughtExceptionHandlerSkipAddressCount;
         i < UncaughtExceptionHandlerSkipAddressCount +
         UncaughtExceptionHandlerReportAddressCount;
         i++)
    {
        [backtrace addObject:[NSString stringWithUTF8String:strs[i]]];
    }
    free(strs);
    
    return backtrace;
}

- (void)validateAndSaveCriticalApplicationData
{

}
- (void)handerReactNativeError:(NSDictionary *)error{
    
    RNLogDO *logDO = [RNLogDO new];
    [logDO setValuesForKeysWithDictionary:error];
    [RNMonitorUploadManager applyBaseParam:KLogEntityName
                                  pageName:nil
                                      data:logDO];
    
}

- (void)handleException:(NSException *)exception
{
    RNLogDO *logDO = [RNLogDO new];
    logDO.erm = [exception reason];
    logDO.sta = [[exception userInfo] objectForKey:UncaughtExceptionHandlerAddressesKey] ;
    
    [RNMonitorUploadManager applyBaseParam:KLogEntityName
                                  pageName:nil
                                      data:logDO];

}




@end
// 异常处理
void HandleException(NSException *exception)
{
    int32_t exceptionCount = OSAtomicIncrement32(&UncaughtExceptionCount);
    if (exceptionCount > UncaughtExceptionMaximum)
    {
        return;
    }
    
    NSArray *callStack = [RNLogManager backtrace];
    NSMutableDictionary *userInfo = [NSMutableDictionary dictionaryWithDictionary:[exception userInfo]];
    [userInfo setObject:callStack forKey:UncaughtExceptionHandlerAddressesKey];
    [[RNLogManager shareInstance] performSelectorOnMainThread:@selector(handleException:)
                                                              withObject:[NSException exceptionWithName:[exception name] reason:[exception reason] userInfo:userInfo]
                                                           waitUntilDone:YES];

}

void SignalHandler(int signal)
{
    int32_t exceptionCount = OSAtomicIncrement32(&UncaughtExceptionCount);
    
    if (exceptionCount > UncaughtExceptionMaximum)
    {
        return;
    }
    
    NSMutableDictionary *userInfo =[NSMutableDictionary dictionaryWithObject:[NSNumber numberWithInt:signal] forKey:UncaughtExceptionHandlerSignalKey];
    
    NSArray *callStack = [RNLogManager backtrace];
    [userInfo setObject:callStack forKey:UncaughtExceptionHandlerAddressesKey];
    
    
    id exception = [NSException exceptionWithName:UncaughtExceptionHandlerSignalExceptionName
                                           reason:[NSString stringWithFormat:NSLocalizedString(@"Signal %d was raised.", nil),signal]
                                         userInfo:[NSDictionary dictionaryWithObject:[NSNumber numberWithInt:signal] forKey:UncaughtExceptionHandlerSignalKey]];
    
    [[RNLogManager shareInstance]  performSelectorOnMainThread:@selector(handleException:)
                                                               withObject:exception
                                                            waitUntilDone:YES];
}

void InstallUncaughtExceptionHandler(void)
{
    NSSetUncaughtExceptionHandler(&HandleException);
    signal(SIGABRT, SignalHandler);
    signal(SIGILL, SignalHandler);
    signal(SIGSEGV, SignalHandler);
    signal(SIGFPE, SignalHandler);
    signal(SIGBUS, SignalHandler);
    signal(SIGPIPE, SignalHandler);
}


