//
//  RNNetWorkFlow.m
//  RNMonitor
//
//  Created by zhiyu on 17/3/8.
//  Copyright © 2017年 Facebook. All rights reserved.
//

#import "RNNetWorkFlow.h"
#include <ifaddrs.h>
#include <net/if.h>
@interface RNNetWorkFlow()

@property (nonatomic, copy) void (^netBlock)(u_int32_t sendFlow, u_int32_t receivedFlow);  //每秒获取

@property (assign,nonatomic) uint32_t historySent;
@property (assign,nonatomic) uint32_t historyRecived;
@property (assign,nonatomic) BOOL     isFirst;
@property (nonatomic, strong)NSTimer *timer;

@end
@implementation RNNetWorkFlow
+ (instancetype)shareInstance {
    static RNNetWorkFlow *_instance;
    static dispatch_once_t once_t;
    dispatch_once(&once_t, ^{
        if (!_instance) {
            _instance = [[self alloc] init];
        }
    });
    return _instance;
}
- (instancetype)init {
    if ([super init]) {
        [self getNetflow];
    }
    
    return self;
}

- (void)dealloc {
    [self.timer invalidate];
    self.timer = nil;
    
    self.netBlock = nil;
}

- (void)startblock:(void (^)(u_int32_t sendFlow, u_int32_t receivedFlow))block {
    self.netBlock = block;
    self.isFirst = YES;
    self.timer = [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(getNetflow) userInfo:nil repeats:YES];
}

- (void)stop {
    [self.timer invalidate];
    self.timer = nil;
}

- (uint32_t)currentNetFlow{
    return self.historySent + self.historyRecived;
}

/** 流量消耗状态 **/
- (void)getNetflow {
    BOOL   success;
    struct ifaddrs *addrs;
    const struct ifaddrs *cursor;
    const struct if_data *networkStatisc;
    
    self.kWiFiSent = 0;
    self.kWiFiReceived = 0;
    self.kWWANSent = 0;
    self.kWWANReceived = 0;
    
    NSString *name = @"";
    
    success = getifaddrs(&addrs) == 0;
    if (success)
    {
        cursor = addrs;
        while (cursor != NULL)
        {
            name=[NSString stringWithFormat:@"%s",cursor->ifa_name];
            
            // names of interfaces: en0 is WiFi ,pdp_ip0 is WWAN
            if (cursor->ifa_addr->sa_family == AF_LINK)
            {
                if ([name hasPrefix:@"en"])
                {
                    networkStatisc = (const struct if_data *) cursor->ifa_data;
                    self.kWiFiSent+=networkStatisc->ifi_obytes;
                    self.kWiFiReceived+=networkStatisc->ifi_ibytes;
                }
                if ([name hasPrefix:@"pdp_ip"])
                {
                    networkStatisc = (const struct if_data *) cursor->ifa_data;
                    self.kWWANSent+=networkStatisc->ifi_obytes;
                    self.kWWANReceived+=networkStatisc->ifi_ibytes;
                }
            }
            cursor = cursor->ifa_next;
        }
        freeifaddrs(addrs);
    }
    
    //第一次不统计
    if (self.isFirst) {
        self.isFirst = NO;
    }
    else {
        uint32_t nowSent = (self.kWiFiSent + self.kWWANSent - self.historySent);
        uint32_t nowRecived = (self.kWiFiReceived + self.kWWANReceived - self.historyRecived);
        
        if (self.netBlock) {
            self.netBlock(nowSent, nowRecived);
        }
    }
    
    self.historySent = self.kWiFiSent + self.kWWANSent;
    self.historyRecived = self.kWiFiReceived + self.kWWANReceived;
}

@end
