//
//  LPCoreDatamanager.m
//  demo
//
//  Created by zhiyu on 17/2/23.
//  Copyright © 2017年 terminus. All rights reserved.
//

#import "LPCoreDatamanager.h"

@interface LPCoreDatamanager()
/**
 *  数据模型对象
 */
@property (nonatomic,strong) NSManagedObjectModel *model;
/**
 *  上下文
 */
@property (nonatomic,strong) NSManagedObjectContext *context;
/**
 *  持久性存储区
 */
@property (nonatomic,strong) NSPersistentStoreCoordinator *persistent;
@end

@implementation LPCoreDatamanager

- (instancetype)initWithCoreData:(NSString *)entityName modelName:(NSString *)modelName sqlPath:(NSString *)sqlPath success:(void(^)(void))success fail:(void(^)(NSError *error))fail
{
    if (self = [super init]) {
 
        _entityName = entityName;
        _modelName = modelName;
        _sqlPath = sqlPath;
        self.context = [[NSManagedObjectContext alloc] initWithConcurrencyType:NSMainQueueConcurrencyType];
        if (modelName) {
            NSURL *modelURL = [[NSBundle mainBundle] URLForResource:@"RNMonitor" withExtension:@"mom"];
            
            self.model = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
        } else {
            self.model = [NSManagedObjectModel mergedModelFromBundles:nil];
        }
        
        self.persistent = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:self.model];
 
        NSError *error = nil;
        [self.persistent addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:[NSURL fileURLWithPath:sqlPath] options:nil error:&error];
        if (error) {
            if (fail) {
                fail(error);
            }
        } else {
            self.context.persistentStoreCoordinator = self.persistent;
            if (success) {
                success();
            }
        }
    }
    
    return self;
}

// 添加数据
- (void)insertNewEntity:(NSDictionary *)dict success:(void(^)(void))success fail:(void(^)(NSError *error))fail
{
    if (!dict||dict.allKeys.count == 0) return;
    NSManagedObject *newEntity = [NSEntityDescription insertNewObjectForEntityForName:self.entityName inManagedObjectContext:self.context];
    for (NSString *key in [dict allKeys]) {
        [newEntity setValue:[dict objectForKey:key] forKey:key];
    }
    NSError *error = nil;
    BOOL result = [self.context save:&error];
    if (!result) {
        if (fail) {
            fail(error);
        }
    } else {
        if (success) {
            success();
        }
    }
}

// 查询数据
- (void)readEntity:(NSArray *)sequenceKeys ascending:(BOOL)isAscending filterStr:(NSString *)filterStr success:(void(^)(NSArray *results))success fail:(void(^)(NSError *error))fail
{
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    NSEntityDescription *desc = [NSEntityDescription entityForName:self.entityName inManagedObjectContext:self.context];
    request.entity = desc;
    if (sequenceKeys&&sequenceKeys.count>0) {
        NSMutableArray *array = [NSMutableArray array];
        for (NSString *key in sequenceKeys) {
           
            NSSortDescriptor *sort = [NSSortDescriptor sortDescriptorWithKey:key ascending:isAscending];
            [array addObject:sort];
        }
        if (array.count>0) {
            request.sortDescriptors = array;
        }
    }
    if (filterStr) {
        NSPredicate *predicate = [NSPredicate predicateWithFormat:filterStr];
        request.predicate = predicate;
    }
    NSError *error = nil;
    NSArray *objs = [self.context executeFetchRequest:request error:&error];
    if (error) {
        if (fail) {
            fail(error);
        }
    } else{
        if (success) {
            success(objs);
        }
    }
}

// 更新数据
- (void)updateEntity:(void(^)(void))success fail:(void(^)(NSError *error))fail
{
    NSError *error = nil;
    [self.context save:&error];
    if (error) {
        if (fail) {
            fail(error);
        }
    } else {
        if (success) {
            success();
        }
    }
    
}

// 删除数据
- (void)deleteEntity:(NSManagedObject *)entity success:(void(^)(void))success fail:(void(^)(NSError *error))fail
{
    [self.context deleteObject:entity];
    NSError *error = nil;
    [self.context save:&error];
    if (error) {
        if (fail) {
            fail(error);
        }
    } else {
        if (success) {
            success();
        }
    }
}
@end
